/*
** This module implements a string class that supports concatenation.
*/
static char const rcsid[] = "@(#) $Header: /home/softools/cvs/softools/code/common/cked/src/ss_string.cpp,v 1.6 2003/01/29 16:09:19 mark Exp $";

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <tcl.h>
#include "ss_string.h"

#if INTERFACE
/*
** object: SsString
**
** A single object of the SsString class represents a string.
** Operations such as setting, prepending, and appending are supported.
*/
class SsString {
protected:
  char *zString; // The string, in allocated memory
  char *zSave;   // Saved copy of zString that can be modified
  int iNext;     // Index in zString where next substring will be retrieved.
                 // -1 if there are no substrings remaining.
public:
  int isnull() {return (!zString || !*zString);}
  virtual SsString::~SsString() {delete [] zString; delete [] zSave;}
  virtual SsString *SsString::append(const char *zStr);
  virtual SsString *SsString::append(int i);
};
#endif

PUBLIC SsString::SsString(){
  zString = 0;
  zSave = 0;
  set("");
}
PUBLIC SsString::SsString(const char *zStr){
  zString = 0;
  zSave = 0;
  set(zStr);
}
PUBLIC SsString::SsString(const SsString &oldStr){
  zString = new char[strlen(oldStr.zString) + 1];
  strcpy(zString, oldStr.zString);
  iNext = -1;
  zSave = 0;
}

/*
** Allow an integer value to be the initializer.
** The SsString will have the string rep of the value.
*/
PUBLIC SsString::SsString(const int value){
  zString = 0;
  zSave = 0;
  set(value);
}

/*
** Set the value of the string object to have string rep zStr.
*/
PUBLIC void SsString::set(const char *zStr){
  if( zString ){
    delete [] zString;
  }
  if( zSave ){
    delete [] zSave;
    zSave = 0;
  }
  if( zStr ){
    zString = new char[strlen(zStr) + 1];
    strcpy(zString, zStr);
  } else {
    zString = new char[1];
    zString[0] = 0;
  }
  iNext = -1;
  zSave = 0;
}

PUBLIC void SsString::set(const int i){
  char zInt[33];           /* large enough for any 32-bit integer */
  sprintf(zInt, "%d", i);
  set(zInt);
}

SsString *SsString::append(const char *zStr){
  if( !zStr || !*zStr ) {
    return this;
  }
  char *zNewString = new char[strlen(zString) + strlen(zStr) + 1];
  strcpy(zNewString, zString);
  strcat(zNewString, zStr);
  delete [] zString;
  zString = zNewString;
  return this;
}

SsString *SsString::append(const int i){
  char zInt[33];           /* large enough for any 32-bit integer */
  sprintf(zInt, "%d", i);
  return append(zInt);
}

/*
** The "+=" operator concatenates the current string object with
** the string referenced by the given character pointer
** A reference to the string object is returned.
*/
PUBLIC SsString *SsString::operator+=(const char *zStr){
  return append(zStr);
}

/*
** Works like the "+=" operator above, but allows the string representation
** of an integer to be concatenated to the string.
** A reference to the string object is returned.
*/
PUBLIC SsString *SsString::operator+=(const int i){
  return append(i);
}

PUBLIC SsString *SsString::prepend(const char *zStr){
  if( !zStr || !*zStr ){
    return this;
  }
  char *zNewString = new char[strlen(zString) + strlen(zStr) + 1];
  strcpy(zNewString, zStr);
  strcat(zNewString, zString);
  delete [] zString;
  zString = zNewString;
  return this;
}

/*
** Return the character string associated with
*/
PUBLIC char *SsString::render(){
  return zString;
}

/*
** Return the first space-separated substring of the string.
*/
PUBLIC char *SsString::first(){

  /* Remove any existing copy of zSave
  */
  if( zSave ){
    delete [] zSave;
    zSave = 0;
  }

  /* Copy zString into zSave, and find the next substring from zSave
   */
  zSave = new char[strlen(zString)+1];
  strcpy(zSave, zString);
  iNext = 0;
  return next();
}

/*
** Return the next space-separated substring of the string, or 0 if there
** are no more.
*/
PUBLIC char *SsString::next(){

  /* If the string has been traversed, clean up and return 0
  */
  if( iNext==-1 ){
    if( zSave ){
      delete [] zSave;
      zSave = 0;
    }
    return 0;
  }

  /* Skip space at the beginning of the string.
  ** If there's nothing left after skipping space, clean up and return 0.
  */
  char *zThis = zSave + iNext;
  while( *zThis==' ' ) ++zThis;
  if( !*zThis ){
    iNext = -1;
    if( zSave ){
      delete [] zSave;
      zSave = 0;
    }
    return 0;
  }

  /* Create the null-terminated token, store the location of the next token,
  ** and return the token.
  */
  char *zCur = zThis;
  while( *zThis && *zThis!=' ' ) ++zThis;

  if( !*zThis ){
    iNext = -1;
  } else {
    *zThis++ = 0;
    iNext = zThis - zSave;
  }

  return zCur;
}
